<?php

namespace Drupal\tone;

use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;

class ToneExtraFields implements ToneExtraFieldsInterface {

  use StringTranslationTrait;

  /**
   * Entity field manager.
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * Constructs a tone extra fields service.
   *
   * @param $entityFieldManager \Drupal\Core\Entity\EntityFieldManagerInterface
   *   The entity field manager service.
   */
  public function __construct(EntityFieldManagerInterface $entityFieldManager) {
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getExtraFieldInfo(): array {
    $extra = [];

    foreach ($this->getFieldMap() as $entity_type_id => $bundles) {
      foreach ($bundles as $bundle => $fields) {
        foreach ($fields as $extra_field_name => $source_field_name) {
          $definition = $this->getExtraFieldDefinition($entity_type_id, $bundle, $source_field_name);
          if (!empty($definition)) {
            $extra[$entity_type_id][$bundle]['display'][$extra_field_name] = $definition;
          }
        }
      }
    }

    return $extra;
  }

  /**
   * {@inheritdoc}
   */
  public function attach(array $build, EntityInterface $entity, string $view_mode, ?EntityViewDisplayInterface $display = NULL): array {
    if (!$display) {
      /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $repository */
      $repository = \Drupal::service('entity_display.repository');
      $display = $repository->getViewDisplay(
        $entity->getEntityTypeId(),
        $entity->bundle(),
        $view_mode
      );
    }

    $map = $this->getFieldMap();

    if (isset($map[$entity->getEntityTypeId()][$entity->bundle()])) {
      $fields = $map[$entity->getEntityTypeId()][$entity->bundle()];
      foreach ($fields as $extra_field_name => $source_field_name) {
        $source_settings = $display->getComponent($source_field_name);
        if ($source_settings && $display->getComponent($extra_field_name)) {
          $view_mode = $source_settings['settings']['view_mode'] ?? 'default';
          foreach ($entity->{$source_field_name} as $child) {
            if ($child->entity instanceof ToneInterface) {
              $build = $child->entity->attach($build, $view_mode);
            }
            else {
              $build = $this->attach($build, $child->entity, $view_mode);
            }
          }
        }
      }
    }

    return $build;
  }

  /**
   * Returns an array of extra field info keyed by entity id.
   *
   * Each entry is an associative array keyed by bundle where the value is
   * a map extra field name => source field name.
   */
  protected function getFieldMap(): array {
    $result = [];

    /** @var \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager */
    foreach ($this->entityFieldManager->getFieldMapByFieldType('entity_reference') as $entity_type => $field_list) {
      foreach ($field_list as $field_name => ['bundles' => $bundles]) {
        foreach ($bundles as $bundle) {
          $result[$entity_type][$bundle]['tone__' . $field_name] = $field_name;
        }
      }
    }

    return $result;
  }

  /**
   * Returns the extra field definition for the given source field.
   */
  protected function getExtraFieldDefinition(string $entity_type_id, string $bundle, string $field_name): array {
    $result = [];

    $definitions = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle);
    if (isset($definitions[$field_name])) {
      $definition = $definitions[$field_name];
      $targs = ['@label' => $definition->getLabel()];

      if ($this->isToneReference($definition->getFieldStorageDefinition())) {
        $result = [
          'label' => $this->t('Tone from @label', $targs),
          'description' => $this->t('Apply a frontend variation from specified tone field to this entity'),
          'weight' => 0,
          'visible' => FALSE,
        ];
      }
      elseif ($this->containsToneReference($definition->getFieldStorageDefinition())) {
        $result = [
          'label' => $this->t('Tones from @label (recursive)', $targs),
          'description' => $this->t('Apply frontend variations from all tone fields referenced by the specified entity to this entity'),
          'weight' => 0,
          'visible' => FALSE,
        ];
      }
    }

    return $result;
  }

  /**
   * Returns TRUE if the given field definition represents a tone entity reference.
   */
  protected function isToneReference(FieldStorageDefinitionInterface $definition): bool {
    return $definition->getType() === 'entity_reference' &&
      $definition->getSetting('target_type') === 'tone';
  }

  /**
   * Returns TRUE if the given field definition is a reference to an entity type
   * containing a tone reference.
   */
  protected function containsToneReference(FieldStorageDefinitionInterface $definition, ?int $maxDepth = NULL): bool {
    $result = FALSE;

    if (!isset($maxDepth)) {
      $maxDepth = Settings::get('tone_recursion_max', 2);
    }

    if ($maxDepth > 0) {
      if ($this->isToneReference($definition)) {
        $result = TRUE;
      }
      elseif ($definition->getType() === 'entity_reference') {
        $targetType = $definition->getSetting('target_type');
        $referenceMap = $this->entityFieldManager->getFieldMapByFieldType('entity_reference');
        if (!empty($referenceMap[$targetType])) {
          $fieldNames = array_keys($referenceMap[$targetType]);
          if (count($fieldNames) > 0) {
            $definitions = $this->entityFieldManager->getFieldStorageDefinitions($targetType);
            foreach ($fieldNames as $fieldName) {
              if (
                isset($definitions[$fieldName]) &&
                $this->containsToneReference($definitions[$fieldName], $maxDepth - 1)
              ) {
                $result = TRUE;
                break;
              }
            }
          }
        }
      }
    }

    return $result;
  }

}
